@@ -1,6 +1,6 @@
 #!/bin/sh
 #Copyright (C) The openNDS Contributors 2004-2022
-#Copyright (C) BlueWave Projects and Services 2015-2023
+#Copyright (C) BlueWave Projects and Services 2015-2024
 #This software is released under the GNU GPL license.
 #
 # Warning - shebang sh is for compatibliity with busybox ash (eg on OpenWrt)
@@ -38,6 +38,10 @@ generate_splash_sequence() {
 header() {
 # Define a common header html for every page served
 	gatewayurl=$(printf "${gatewayurl//%/\\x}")
+	htmlentitydecode "$logo_message"
+	urldecode "$entitydecoded"
+	logo_message="$urldecoded"
+
 	echo "<!DOCTYPE html>
 		<html>
 		<head>
@@ -56,7 +60,7 @@ header() {
 			$gatewayname <br>
 		</med-blue>
 		<div class=\"insert\" style=\"max-width:100%;\">
-		<img src=\"$logo\" alt=\"Placeholder: Logo.\"><br>
+		<img src=\"$gatewayurl""$logo\" alt=\"Placeholder: Logo.\"><br>
 		<b>$logo_message</b><br>
 	"
 }
@@ -68,7 +72,7 @@ footer() {
 		<hr>
 		<div style=\"font-size:0.5em;\">
 			<br>
-			<img style=\"height:60px; width:60px; float:left;\" src=\"$gatewayurl""$imagepath\" alt=\"Splash Page: For access to the Internet.\">
+			<img style=\"height:60px; float:left;\" src=\"$gatewayurl""$logo\" alt=\"Splash Page: For access to the Internet.\">
 			&copy; Portal: BlueWave Projects and Services 2015 - $year<br>
 			<br>
 			Portal Version: $version
@@ -105,9 +109,13 @@ name_email_login() {
 login_form() {
 	# Define a login form
 
+	htmlentitydecode "$banner1_message"
+	urldecode "$entitydecoded"
+	banner1_message="$urldecoded"
+
 	echo "
 		<big-red>Welcome!</big-red><br>
-		<img style=\"width:100%; max-width: 100%;\" src=\"$banner1\" alt=\"Placeholder: Banner1.\"><br>
+		<img style=\"width:100%; max-width: 100%;\" src=\"$gatewayurl""$banner1\" alt=\"Placeholder: Banner1.\"><br>
 		<b>$banner1_message</b><hr>
 		<med-blue>You are connected to $client_zone</med-blue><br>
 		<italic-black>
@@ -116,8 +124,8 @@ login_form() {
 		<hr>
 		<form action=\"/opennds_preauth/\" method=\"get\">
 			<input type=\"hidden\" name=\"fas\" value=\"$fas\">
-			<input type=\"text\" name=\"username\" value=\"$username\" autocomplete=\"on\" ><br>Name<br><br>
-			<input type=\"email\" name=\"emailaddress\" value=\"$emailaddress\" autocomplete=\"on\" ><br>Email<br><br>
+			<input type=\"text\" name=\"username\" value=\"$username\" autocomplete=\"on\" ><br><b>Name</b><br><br>
+			<input type=\"email\" name=\"emailaddress\" value=\"$emailaddress\" autocomplete=\"on\" ><br><b>Email</b><br><br>
 			$custom_inputs
 			<input type=\"submit\" value=\"Accept Terms of Service\" >
 		</form>
@@ -141,6 +149,12 @@ thankyou_page () {
 	# Be aware that many devices will close the login browser as soon as
 	# the client user continues, so now is the time to deliver your message.
 
+	htmlentitydecode "$banner2_message"
+	urldecode "$entitydecoded"
+	banner2_message="$urldecoded"
+
+urldecode "$test_variable2"
+test_variable2="$urldecoded"
 	echo "
 		<big-red>
 			Thankyou for using this service.<br>Please click Continue for access.
@@ -148,7 +162,7 @@ thankyou_page () {
 		<br>
 		<b>Welcome $username</b>
 		<br>
-		<med-blue>You are connected to $client_zone</med-blue><br>
+		<med-blue>You are connected to <br>$client_zone</med-blue><br>
 	"
 
 	# Add your message here:
@@ -164,7 +178,7 @@ thankyou_page () {
 	echo "
 		<br>
 		<italic-black>
-			<img style=\"width:100%; max-width: 100%;\" src=\"$banner2\" alt=\"Placeholder: Banner2.\"><br>
+			<img style=\"width:100%; max-width: 100%;\" src=\"$gatewayurl""$banner2\" alt=\"Placeholder: Banner2.\"><br>
 			<b>$banner2_message</b><br>
 			$advert1
 			<hr>
@@ -214,13 +228,18 @@ landing_page() {
 
 	# output the landing page - note many CPD implementations will close as soon as Internet access is detected
 	# The client may not see this page, or only see it briefly
+
+	htmlentitydecode "$banner3_message"
+	urldecode "$entitydecoded"
+	banner3_message="$urldecoded"
+
 	auth_success="
 		<p>
 			<big-red>
 				You are now logged in and have been granted access to the Internet.
 			</big-red>
 			<hr>
-			<img style=\"width:100%; max-width: 100%;\" src=\"$banner3\" alt=\"Placeholder: Banner1.\"><br>
+			<img style=\"width:100%; max-width: 100%;\" src=\"$gatewayurl""$banner3\" alt=\"Placeholder: Banner3.\"><br>
 			<b>$banner3_message</b><br>
 		</p>
 		<hr>
@@ -229,7 +248,6 @@ landing_page() {
 				You can use your Browser, Email and other network Apps as you normally would.
 			</italic-black>
 
-			(Your device originally requested $originurl)
 			<hr>
 			Click or tap Continue to show the status of your account.
 		</p>
@@ -244,7 +262,7 @@ landing_page() {
 				Something went wrong and you have failed to log in.
 			</big-red>
 			<hr>
-			<img style=\"width:100%; max-width: 100%;\" src=\"$banner3\" alt=\"Placeholder: Banner1.\"><br>
+			<img style=\"width:100%; max-width: 100%;\" src=\"$banner3\" alt=\"Placeholder: Banner3.\"><br>
 			<b>$banner3_message</b><br>
 		</p>
 
