@@ -1,5 +1,5 @@
 <?php
-/* (c) Blue Wave Projects and Services 2015-2023. This software is released under the GNU GPL license.
+/* (c) Blue Wave Projects and Services 2015-2024. This software is released under the GNU GPL license.
 
  This is a FAS script providing an example of remote Forward Authentication for openNDS (NDS) on an http web server supporting PHP.
 
@@ -18,8 +18,8 @@
 
  5. faskey: Matching $key as set in this script (see below this introduction).
 	This is a key phrase for NDS to encrypt the query string sent to FAS.
-	It can be any combination of A-Z, a-z and 0-9, up to 16 characters with no white space.
-	eg 1234567890
+	It can be any combination of A-Z, a-z and 0-9, up to 64 characters with no white space.
+	eg c775e7b757ede630cd0aa1113bd102661ab38829ca52a6422ab782862f268646
 
  6. fas_secure_enabled:  set to level 3
 	The NDS parameters: clientip, clientmac, gatewayname, client token, gatewayaddress, authdir and originurl
@@ -55,8 +55,7 @@
 */
 
 // Set the pre-shared key. This **MUST** be the same as faskey in the openNDS config.
-//	Note: the default value of 1234567890 will not be accepted.
-//	For example, use the sha256 hash of a secret string, but note also that if not present in the openNDS config, one will be generated and added.
+//	For example, use the sha256 hash of a secret string, but note also that if not present in the openNDS config, one will be generated and added there.
 //	See: /etc/config/opennds on the router.
 
 $key="c775e7b757ede630cd0aa1113bd102661ab38829ca52a6422ab782862f268646";
