@@ -22,7 +22,7 @@
   @brief Config file parsing
   @author Copyright (C) 2004 Philippe April <papril777@yahoo.com>
   @author Copyright (C) 2007 Paul Kube <nodogsplash@kokoro.ucsd.edu>
-  @author Copyright (C) 2015-2023 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
+  @author Copyright (C) 2015-2024 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
  */
 
 #define _GNU_SOURCE
@@ -191,7 +191,7 @@ config_init(int argc, char **argv)
 
 	// Special handling for gatewayname as library call returns a url-encoded response
 	gatewayname_raw = safe_calloc(SMALL_BUF);
-	gatewayname = safe_calloc(SMALL_BUF);
+
 	gatewayname = safe_strdup(set_option_str("gatewayname", DEFAULT_GATEWAYNAME, debug_level));
 	uh_urldecode(gatewayname_raw, SMALL_BUF, gatewayname, SMALL_BUF);
 	config.gw_name = safe_strdup(gatewayname_raw);
@@ -302,6 +302,8 @@ config_init(int argc, char **argv)
 	// Lists
 	parse_trusted_mac_list(set_list_str("trustedmac", DEFAULT_TRUSTEDMACLIST, debug_level));
 	parse_fas_custom_parameters_list(set_list_str("fas_custom_parameters_list", DEFAULT_FAS_CUSTOM_PARAMETERS_LIST, debug_level));
+	parse_fas_custom_images_list(set_list_str("fas_custom_images_list", DEFAULT_FAS_CUSTOM_IMAGES_LIST, debug_level));
+	parse_fas_custom_files_list(set_list_str("fas_custom_files_list", DEFAULT_FAS_CUSTOM_FILES_LIST, debug_level));
 
 	// Before we do anything else, reset the firewall (cleans it, in case we are restarting or after an opennds crash)
 	iptables_fw_destroy();
@@ -345,7 +347,7 @@ config_init(int argc, char **argv)
 	debug(LOG_NOTICE, "Interface %s is at %s (%s)", config.gw_interface, config.gw_ip, config.gw_mac);
 
 	// Make sure fas_remoteip is set. Note: This does not enable FAS.
-	if (strcmp(config.fas_remoteip, "") == 0) {
+	if (strcmp(config.fas_remoteip, "disabled") == 0) {
 		config.fas_remoteip = safe_strdup(config.gw_ip);
 	}
 
@@ -578,6 +580,7 @@ void parse_trusted_mac_list(const char ptr[])
 		}
 	}
 
+	free(ptrcopy);
 	free(ptrcopyptr);
 }
 
