@@ -24,7 +24,7 @@
   @author Copyright (C) 2004 Philippe April <papril777@yahoo.com>
   @author Copyright (C) 2004 Alexandre Carmel-Veilleux <acv@miniguru.ca>
   @author Copyright (C) 2008 Paul Kube <nodogsplash@kokoro.ucsd.edu>
-  @author Copyright (C) 2015-2023 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
+  @author Copyright (C) 2015-2024 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
  */
 
 
@@ -205,10 +205,14 @@ termination_handler(int s)
 	free(msg);
 
 	// Restart dnsmasq
-	safe_asprintf(&dnscmd, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" &");
+	dnscmd = safe_calloc(STATUS_BUF);
+	safe_snprintf(dnscmd, STATUS_BUF, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" ");
 	debug(LOG_DEBUG, "restart command [ %s ]", dnscmd);
-	system(dnscmd);
-	debug(LOG_INFO, "Dnsmasq restarted");
+	if (system(dnscmd) == 0) {
+		debug(LOG_INFO, "Dnsmasq restarted");
+	} else {
+		debug(LOG_ERR, "Dnsmasq restart failed!");
+	}
 	free(dnscmd);
 
 	auth_client_deauth_all();
@@ -379,7 +383,6 @@ setup_from_config(void)
 	int routercheck;
 
 	// Initialise config->ext_gateway and check router config
-	config->ext_gateway = safe_calloc(SMALL_BUF);
 	routercheck = check_routing(watchdog);
 
 	// Warn if Preemptive Authentication is enabled
@@ -512,15 +515,16 @@ setup_from_config(void)
 		free(msg);
 	}
 
-	// Restart dnsmasq (because we need it to resolve our gateway fqdn) and wait for it
+	// Reload dnsmasq (because we need it to resolve our gateway fqdn) and wait for it
 	dnscmd = safe_calloc(STATUS_BUF);
-	safe_snprintf(dnscmd, STATUS_BUF, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" ");
-	debug(LOG_DEBUG, "restart command [ %s ]", dnscmd);
-	system(dnscmd);
-	debug(LOG_INFO, "Dnsmasq restarting");
+	safe_snprintf(dnscmd, STATUS_BUF, "/usr/lib/opennds/dnsconfig.sh \"reload_only\" ");
+	debug(LOG_DEBUG, "reload command [ %s ]", dnscmd);
+	if (system(dnscmd) == 0) {
+		debug(LOG_INFO, "Dnsmasq reloading");
+	} else {
+		debug(LOG_ERR, "Dnsmasq reload failed!");
+	}
 	free(dnscmd);
-	// Even though the command completed, dnsmasq might not be up yet, so sleep a little
-	sleep(1);
 
 	// Encode gatewayname
 	char idbuf[STATUS_BUF] = {0};
@@ -607,6 +611,10 @@ setup_from_config(void)
 		debug(LOG_INFO, "fas_secure_enabled is set to level %d", config->fas_secure_enabled);
 
 		// Check the FAS remote IP address
+		if ((strcmp(config->fas_remoteip, "disabled") == 0)) {
+			config->fas_remoteip = safe_strdup(config->gw_ip);
+		}
+
 		if (config->fas_remoteip) {
 			if (is_addr(config->fas_remoteip) == 1) {
 				debug(LOG_INFO, "fasremoteip - %s - is a valid IPv4 address...", config->fas_remoteip);
@@ -620,7 +628,7 @@ setup_from_config(void)
 		// Block fas port 80 if local FAS
 		snprintf(port, sizeof(port), "%u", config->fas_port);
 
-		if((strcmp(config->gw_ip, config->fas_remoteip) == 0) && (strcmp(port, "80") == 0)) {
+		if ((strcmp(config->gw_ip, config->fas_remoteip) == 0) && (strcmp(port, "80") == 0)) {
 			debug(LOG_ERR, "Invalid fasport - port 80 is reserved and cannot be used for local FAS...");
 			debug(LOG_ERR, "Exiting...");
 			exit(1);
@@ -719,7 +727,9 @@ setup_from_config(void)
 
 		// Check if authmon is running and if it is, kill it
 		safe_asprintf(&fasssl, "kill $(pgrep -f \"usr/lib/opennds/authmon.sh\") > /dev/null 2>&1");
-		system(fasssl);
+		if (system(fasssl) < 0) {
+			debug(LOG_ERR, "Error returned from system call - Continuing");
+		}
 		free(fasssl);
 
 		// Start the authmon daemon if configured for Level >= 3
@@ -751,7 +761,9 @@ setup_from_config(void)
 
 			debug(LOG_DEBUG, "authmon startup command is: %s\n", fasssl);
 
-			system(fasssl);
+			if (system(fasssl) != 0) {
+				debug(LOG_ERR, "Error returned from system call - Continuing");
+			}
 			free(fasssl);
 
 			// Check authmon is running
@@ -875,12 +887,16 @@ setup_from_config(void)
 
 	free(msg);
 
-	// Restart dnsmasq again for nftsets, but this time se can do it in the background
+	// Reload dnsmasq again for nftsets, but this time we can do it in the background
 	dnscmd = safe_calloc(STATUS_BUF);
-	safe_snprintf(dnscmd, STATUS_BUF, "/usr/lib/opennds/dnsconfig.sh \"restart_only\" &");
-	debug(LOG_DEBUG, "restart command [ %s ]", dnscmd);
-	system(dnscmd);
-	debug(LOG_INFO, "Dnsmasq restarting");
+	safe_snprintf(dnscmd, STATUS_BUF, "/usr/lib/opennds/dnsconfig.sh \"reload_only\" &");
+	debug(LOG_DEBUG, "reload command [ %s ]", dnscmd);
+	if (system(dnscmd) == 0) {
+		debug(LOG_INFO, "Dnsmasq reloading");
+	} else {
+		debug(LOG_ERR, "Dnsmasq reload failed!");
+	}
+
 	free(dnscmd);
 
 
@@ -927,7 +943,9 @@ main_loop(int argc, char **argv)
 
 	debug(LOG_NOTICE, "openNDS is now running.\n");
 	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh \"auth_restore\" &");
-	system(cmd);
+	if (system(cmd) != 0) {
+		debug(LOG_ERR, "failure: %s", cmd);
+	}
 	free(cmd);
 
 	result = pthread_join(tid, NULL);
