@@ -24,8 +24,8 @@
   @author Copyright (C) 2004 Philippe April <papril777@yahoo.com>
   @author Copyright (C) 2006 Benoit Grégoire <bock@step.polymtl.ca>
   @author Copyright (C) 2008 Paul Kube <nodogsplash@kokoro.ucsd.edu>
-  @author Copyright (C) 2015-2023 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
   @author Copyright (C) 2021 ndsctl_lock() and ndsctl_unlock() based on code by Linus Lüssing <ll@simonwunderlich.de>
+  @author Copyright (C) 2015-2024 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
 
  */
 
@@ -342,6 +342,10 @@ int check_routing(int watchdog)
 			}
 		}
 
+		if (config->ext_gateway) {
+			free (config->ext_gateway);
+		}
+
 		config->ext_gateway = safe_strdup(rtest);
 		free (rcmd);
 		free (rtest);
@@ -350,6 +354,9 @@ int check_routing(int watchdog)
 	} else {
 		debug(LOG_ERR, "Unable to get routing configuration, retrying later ...");
 		config->online_status = 0;
+		free (rcmd);
+		free (rtest);
+
 		return config->online_status;
 	}
 }
